/*
 *  SingleDouble.c
 *
 *  This program demonstrates double buffering for 
 *  flicker-free animation.  The left and right mouse
 *  buttons start and stop the spinning motion of the square.
 */

#include <GL/glut.h>
#include <stdio.h>

static GLfloat spin = 0.0;
int singleb, doubleb;

/* single-buffer display callback */
void displayd (void)
{
    glClear (GL_COLOR_BUFFER_BIT);
    glRectf (-25.0, -25.0, 25.0, 25.0);
    glutSwapBuffers ();
}

/* double-buffer display callback */
void displays (void)
{
    glClear (GL_COLOR_BUFFER_BIT);
    glRectf (-25.0, -25.0, 25.0, 25.0);
    glFlush ();
}

/* spin both squares, in both windows */
void spinDisplay (void)
{
    spin += 1.0;
    if (spin > 360.0)
        spin = spin - 360.0;

    glutSetWindow (singleb);
    glLoadIdentity ();
    glRotatef (spin, 0.0, 0.0, 1.0);
    glutPostRedisplay (); 

    glutSetWindow (doubleb);
    glLoadIdentity ();
    glRotatef (spin, 0.0, 0.0, 1.0);
    glutPostRedisplay ();
}

/* initialization */
void myinit (void)
{
    glClearColor (0.0, 0.0, 0.0, 1.0);
    glColor3f (1.0, 1.0, 1.0);
    glShadeModel (GL_FLAT);
}

/* handle left and right mouse presses */
void mouse (int btn, int state, int x, int y)
{
    if (btn == GLUT_LEFT_BUTTON  && state == GLUT_DOWN)
        glutIdleFunc (spinDisplay);

    if (btn == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
        glutIdleFunc (NULL);
}

/* handle reshaping of a window */
void myReshape (int w, int h)
{
    glViewport (0, 0, w, h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    if (w <= h) 
        glOrtho (-50.0, 50.0, -50.0 * (GLfloat) h / (GLfloat) w, 
                               50.0 * (GLfloat) h / (GLfloat) w, -1.0, 1.0);
    else 
        glOrtho (-50.0 * (GLfloat) w / (GLfloat) h, 
                  50.0 * (GLfloat) w / (GLfloat) h, -50.0, 50.0, -1.0, 1.0);
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
}

/*  Main Loop
 *  Open window with initial window size, title bar, 
 *  RGBA display mode, and handle input events.
 */

int main (int argc, char** argv)
{
    printf ("\nLeft mouse starts animation; right mouse stops animation\n\n");
    glutInit (&argc, argv);

    /* set up single buffer window */
    glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
    glutInitWindowSize (300, 300);
    glutInitWindowPosition (25, 50);
    singleb = glutCreateWindow ("single buffered");
    myinit ();
    glutDisplayFunc (displays); 
    glutReshapeFunc (myReshape);
    glutIdleFunc (spinDisplay);
    glutMouseFunc (mouse);

    /* set up double buffer window */
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize (300, 300);
    glutInitWindowPosition (340, 50);
    doubleb = glutCreateWindow ("double buffered");
    myinit ();
    glutDisplayFunc (displayd);
    glutReshapeFunc (myReshape);
    glutIdleFunc (spinDisplay);
    glutMouseFunc (mouse);

    glutMainLoop ();

    return 0;
}
